from django.contrib import admin
from story_map.models import Story, Slide


@admin.register(Slide)
class RecogitoAdmin(admin.ModelAdmin):
    list_display = (
        "order_nr",
        "story",
        "text_headline",
        "text_text",
        "location_lat",
        "location_lng",
        "media_url"
    )
    list_filter = (
        "story",
    )


@admin.register(Story)
class SlideAdmin(admin.ModelAdmin):
    pass



# Generated by Django 3.2.11 on 2022-01-27 12:45

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Story',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=250)),
                ('attribution', models.CharField(blank=True, max_length=250, null=True)),
                ('call_to_action', models.BooleanField(default=True)),
                ('call_to_action_text', models.CharField(blank=True, max_length=250, null=True)),
                ('language', models.CharField(default='en', max_length=2)),
                ('map_background_color', models.CharField(default='#5f9468', max_length=15)),
                ('map_subdomains', models.CharField(blank=True, max_length=25, null=True)),
                ('map_type', models.CharField(blank=True, max_length=25, null=True)),
                ('zoomify_path', models.CharField(blank=True, max_length=250, null=True)),
                ('zoomify_height', models.PositiveSmallIntegerField(blank=True, null=True)),
                ('zoomify_width', models.PositiveSmallIntegerField(blank=True, null=True)),
                ('zoomify_attribution', models.CharField(blank=True, max_length=250, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='Slide',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('order_nr', models.PositiveSmallIntegerField(help_text='Use this number for ordering your slides')),
                ('location_lat', models.FloatField(blank=True, null=True)),
                ('location_lng', models.FloatField(blank=True, null=True)),
                ('location_zoom', models.PositiveSmallIntegerField(default=4)),
                ('location_icon_size_l', models.PositiveSmallIntegerField(default=48)),
                ('location_icon_size_w', models.PositiveSmallIntegerField(default=48)),
                ('location_line', models.BooleanField(default=True)),
                ('text_headline', models.CharField(max_length=250)),
                ('text_text', models.TextField(blank=True, null=True)),
                ('date', models.DateField(blank=True, null=True)),
                ('media_caption', models.CharField(blank=True, max_length=250, null=True)),
                ('media_credit', models.CharField(blank=True, max_length=250, null=True)),
                ('media_url', models.URLField(blank=True, max_length=250, null=True)),
                ('story', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='has_slides', to='story_map.story')),
            ],
            options={
                'ordering': ['order_nr'],
            },
        ),
    ]



from django.http import JsonResponse
from django.views.generic.detail import DetailView

from . models import Story


class StoryDetailView(DetailView):
    model = Story
    template_name = 'story_map/story_detail.html'


class StoryJsonData(DetailView):
    model = Story

    def render_to_response(self, context, **kwargs):
        data = self.object.payload
        return JsonResponse(data)



from django.urls import path
from . import views

app_name = 'story_map'

urlpatterns = [
    path('<int:pk>/', views.StoryDetailView.as_view()),
    path('data/<int:pk>/', views.StoryJsonData.as_view()),
]



import requests
from story_map.models import Slide, Story


def fetch_data_from_knightlab(url):
    r = requests.get(url)
    data = r.json()
    return data['storymap']


def enrich_story(story, story_data):
    for x in [
        'attribution',
        'call_to_action',
        'language',
        'map_background_color',
        'map_subdomains',
        'map_type',
    ]:
        try:
            setattr(story, x, story_data[x])
        except KeyError:
            pass
    if story_data['zoomify']:
        for x in [
            'path', 'height', 'width', 'attribution'
        ]:
            setattr(story, f"zoomify_{x}", story_data['zoomify'][x])
    story.save()
    return story


def save_slides(data, story_title='default story'):
    story, _ = Story.objects.get_or_create(title=story_title)
    story.has_slides.all().delete()
    enrich_story(story, data)
    items = []
    for i, x in enumerate(data['slides']):
        order_nr = i + 1
        item = Slide.objects.create(
            story=story,
            order_nr=order_nr,
            text_headline=x['text']['headline'],
            media_caption=x['media']['caption'],
            media_credit=x['media']['credit'],
            media_url=x['media']['url'],
        )
        try:
            item.text_text = x['text']['text']
        except KeyError:
            pass
        try:
            item.location_lat = x['location']['lat']
            item.location_lng = x['location']['lon']
        except KeyError:
            pass
        item.save()
        items.append(item)
    return(items)


def import_from_knlab(url, story_title):
    data = fetch_data_from_knightlab(url)
    items = save_slides(data, story_title)
    return items



from django.core.management.base import BaseCommand
from story_map.import_utils import import_from_knlab


class Command(BaseCommand):
    help = 'Imports Story Map data from knightlab URL'

    def add_arguments(self, parser):

        parser.add_argument(
            '--url',
            default="https://uploads.knightlab.com/storymapjs/ee3762a02e5862af8f02906ab1d35ec6/mapping-medieval-people-alternative-map/draft.json",  # noqa: E501
            help='URL to fetch data',
        )
        parser.add_argument(
            '--story',
            default="Default Story Map",
            help='Title of the Story Map',
        )

    def handle(self, *args, **kwargs):
        url = kwargs['url']
        story_title = kwargs['story']
        msg = f"Downloading data from URL: {url} for story map {story_title}"
        self.stdout.write(self.style.NOTICE(f"{msg}"))
        slides = import_from_knlab(url, story_title)
        self.stdout.write(self.style.SUCCESS(f"created {len(slides)} slides"))



import json

from django.db import models
from functools import cached_property

# Create your models here.


def xstr(s):
    if s is None:
        return ''
    return str(s)


class Story(models.Model):
    title = models.CharField(
        max_length=250
    )
    attribution = models.CharField(
        blank=True,
        null=True,
        max_length=250
    )
    call_to_action = models.BooleanField(
        default=True
    )
    call_to_action_text = models.CharField(
        blank=True,
        null=True,
        max_length=250
    )
    language = models.CharField(
        default="en",
        max_length=2
    )
    map_background_color = models.CharField(
        default="#5f9468",
        max_length=15
    )
    map_subdomains = models.CharField(
        blank=True,
        null=True,
        max_length=25
    )
    map_type = models.CharField(
        blank=True,
        null=True,
        max_length=25
    )
    zoomify_path = models.CharField(
        blank=True,
        null=True,
        max_length=250
    )
    zoomify_height = models.PositiveSmallIntegerField(
        blank=True,
        null=True
    )
    zoomify_width = models.PositiveSmallIntegerField(
        blank=True,
        null=True
    )
    zoomify_attribution = models.CharField(
        blank=True,
        null=True,
        max_length=250
    )

    def __str__(self):
        return f"{self.title}"

    @cached_property
    def payload(self):
        item = {
            'slides': [],
            'zoomify': {
                'path': xstr(self.zoomify_path),
                'height': self.zoomify_height,
                'width': self.zoomify_width,
                'attribution': xstr(self.attribution)
            },
            'attribution': xstr(self.attribution),
            'call_to_action': self.call_to_action,
            'call_to_action_text': xstr(self.call_to_action_text),
            'language': self.language,
            'map_as_image': False,
            'map_background_color': self.map_background_color,
            'map_subdomains': xstr(self.map_subdomains),
            'map_type': xstr(self.map_type)
        }
        for x in self.has_slides.all():
            item['slides'].append(x.payload)
        return {
            "storymap": item
        }

    @cached_property
    def payload_as_json(self):
        return json.dumps(self.payload)


class Slide(models.Model):
    story = models.ForeignKey(
        'Story',
        on_delete=models.CASCADE,
        related_name="has_slides"
    )
    order_nr = models.PositiveSmallIntegerField(
        help_text="Use this number for ordering your slides"
    )
    location_lat = models.FloatField(
        blank=True,
        null=True
    )
    location_lng = models.FloatField(
        blank=True,
        null=True
    )
    location_zoom = models.PositiveSmallIntegerField(
        default=4
    )
    location_icon_size_l = models.PositiveSmallIntegerField(
        default=48
    )
    location_icon_size_w = models.PositiveSmallIntegerField(
        default=48
    )
    location_line = models.BooleanField(
        default=True
    )
    text_headline = models.CharField(
        max_length=250
    )
    text_text = models.TextField(
        blank=True,
        null=True,
    )
    date = models.DateField(
        blank=True,
        null=True
    )
    media_caption = models.CharField(
        blank=True,
        null=True,
        max_length=250
    )
    media_credit = models.CharField(
        blank=True,
        null=True,
        max_length=250
    )
    media_url = models.URLField(
        blank=True,
        null=True,
        max_length=250
    )

    class Meta:

        ordering = [
            'order_nr',
        ]

    def save(self, *args, **kwargs):
        if self.media_url:
            if self.media_url.startswith('//'):
                self.media_url = self.media_url.replace('//', 'https://')
        super(Slide, self).save(*args, **kwargs)

    def __str__(self):
        return f"{self.story.title}__{self.order_nr} {self.text_headline}"

    @property
    def location(self):
        if self.location_lat:
            item = {
                "iconSize": [
                    self.location_icon_size_w, self.location_icon_size_l
                ],
                "lat": self.location_lat,
                "line": self.location_line,
                "lon": self.location_lng,
                "zoom": self.location_zoom
            }
        else:
            item = {
                "line": self.location_line,
            }
        return item

    @property
    def text(self):
        item = {
            "headline": self.text_headline,
            "text": self.text_text
        }
        return item

    @property
    def payload(self):
        if self.date:
            date = f"{self.date}"
        else:
            date = ""
        item = {
            "background": {},
            "date": f"{date}",
            "location": self.location,
            "media": {
                "caption": self.media_caption,
                "credit": self.media_credit,
                "url": self.media_url
            },
            "text": self.text
        }
        if self.location_lng:
            return item
        else:
            item['type'] = "overview"
            return item
